
/*
libfawk - A function-only AWK dialect - compacted, single-source-file version
(for a human readable source code please visit the project page)

Copyright (c) 2017..2020 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#ifndef fawk_malloc
#define fawk_malloc(ctx,size) malloc(size)
#define fawk_calloc(ctx,n,m) calloc(n, m)
#define fawk_realloc(ctx,ptr,size) realloc(ptr, size)
#define fawk_free(ctx,ptr) free(ptr)
#endif
#ifndef fawk_assert
#include <assert.h>
#define fawk_assert assert
#endif
#ifndef fawk_no_math
#include <math.h>
#define fawk_strtod(str) strtod(str, NULL)
#define fawk_fmod(x,y) fmod(x, y)
#define FAWK_NUM_PRINTF_FMT "%g"
#endif
static void fawk_sym_get_ref(fawk_ctx_t *ctx, const char *fname, int argc, fawk_cell_t *retval)
{
 fawk_cell_t *s;
 if ((argc != 1) || (FAWK_CFUNC_ARG(0)->type != FAWK_STR))
  return;
 s = fawk_sym_lookup(ctx, FAWK_CFUNC_ARG(0)->data.str->str);
 if (s == NULL)
  return;
 if (s->type != FAWK_SYMREF) {
  retval->type = FAWK_SYMREF;
  retval->data.symref.is_local = 0;
  retval->data.symref.ref.global = s;
 }
 else
  *retval = *s;
}
static void fawk_sym_get(fawk_ctx_t *ctx, const char *fname, int argc, fawk_cell_t *retval)
{
 fawk_cell_t *s;
 if ((argc != 1) || (FAWK_CFUNC_ARG(0)->type != FAWK_STR))
  return;
 s = fawk_sym_lookup(ctx, FAWK_CFUNC_ARG(0)->data.str->str);
 if (s == NULL)
  return;
 if (s->type == FAWK_STR) s->data.str->refco++;
 else if (s->type == FAWK_ARRAY) s->data.arr->refco++;
 *retval = *s;
}
FAWK_API int fawk_sym_init(fawk_ctx_t *ctx)
{
 fawk_pkg_t *pkg;
 pkg = fawk_calloc(ctx, sizeof(fawk_pkg_t), 1);
 if (pkg == NULL) return -1;
 pkg->next = ctx->pkg_head;
 ctx->pkg_head = pkg;
 fawk_symtab_regcfunc(ctx, "sym_get_ref", fawk_sym_get_ref);
 fawk_symtab_regcfunc(ctx, "sym_get", fawk_sym_get);
 return 0;
}
